@@ -22,7 +22,7 @@ package Proc::Daemon;
 use strict;
 use POSIX();
 
-$Proc::Daemon::VERSION = '0.17';
+$Proc::Daemon::VERSION = '0.18';
 
 
 ################################################################################
@@ -31,6 +31,7 @@ $Proc::Daemon::VERSION = '0.17';
 #
 #   %Daemon_Settings are hash key=>values and can be:
 #     work_dir     => '/working/daemon/directory'   -> defaults to '/'
+#     setgid       => 12345                         -> defaults to <undef>
 #     setuid       => 12345                         -> defaults to <undef>
 #     child_STDIN  => '/path/to/daemon/STDIN.file'  -> defautls to '</dev/null'
 #     child_STDOUT => '/path/to/daemon/STDOUT.file' -> defaults to '+>/dev/null'
@@ -228,6 +229,12 @@ sub Init {
                     }
                 }
 
+                # Sets the real group identifier and the effective group
+                # identifier for the daemon process before opening files.
+                # Must set group first because you cannot change group
+                # once you have changed user
+                POSIX::setgid( $self->{setgid} ) if defined $self->{setgid};
+
                 # Sets the real user identifier and the effective user
                 # identifier for the daemon process before opening files.
                 POSIX::setuid( $self->{setuid} ) if defined $self->{setuid};
@@ -323,6 +330,7 @@ sub Init {
     if ( $FH_MEMORY ) {
         seek( $FH_MEMORY, 0, 0 );
         @pid = map { chomp $_; $_ eq '' ? undef : $_ } <$FH_MEMORY>;
+        $_ = (/^(\d+)$/)[0] foreach @pid; # untaint
         close $FH_MEMORY;
     }
     elsif ( $self->{memory}{pid_file} ) {
@@ -550,8 +558,8 @@ sub get_pid {
 
     if ( $string ) {
         # $string is already a PID.
-        if ( $string =~ /^\d+$/ ) {
-            $pid = $string;
+        if ( $string =~ /^(\d+)$/ ) {
+            $pid = $1; # untaint
         }
         # Open the pidfile and get the PID from it.
         elsif ( open( my $FH_MEMORY, "<", $string ) ) {
@@ -559,6 +567,7 @@ sub get_pid {
             close $FH_MEMORY;
 
             die "I found no valid PID ('$pid') in the pidfile: '$string'" if $pid =~ /\D/s;
+            $pid = ($pid =~ /^(\d+)$/)[0]; # untaint
 
             $pidfile = $string;
         }
@@ -576,8 +585,12 @@ sub get_pid {
             $pid = <$FH_MEMORY>;
             close $FH_MEMORY;
 
-            if ( ! $pid || ( $pid && $pid =~ /\D/s ) ) { $pid = undef }
-            else { $pidfile = $self->{pid_file} }
+            if ($pid && $pid =~ /^(\d+)$/) {
+                $pid = $1; # untaint
+                $pidfile = $self->{pid_file};
+            } else {
+                $pid = undef;
+            }
         }
 
         # Try to get the PID out of the system process
